package br.usp.ime.prolog;

import org.eclipse.ui.plugin.*;
import org.eclipse.core.runtime.*;
import org.eclipse.core.resources.*;
import org.eclipse.jface.resource.*;
import org.eclipse.jface.preference.IPreferenceStore;
import java.util.*;

import br.usp.ime.prolog.preferences.IPrologPreferenceFields;
import br.usp.ime.prolog.editor.IPrologColorConstants;
import java.io.File;


/**
 * @author Juliana 
 * 
 * The main plugin class to be used in the desktop.
 */
public class PrologPlugin extends AbstractUIPlugin {
	//The shared instance.
	private static PrologPlugin plugin;
	//Resource bundle.
	private ResourceBundle resourceBundle;		
	// Prolog extensions
	public static String[] PROLOG_EXTENSIONS = {"pro", "pl"};
	
		
	/**
	 * The constructor.
	 */
	public PrologPlugin(IPluginDescriptor descriptor) {
		super(descriptor);
		plugin = this;
		try {
			resourceBundle= ResourceBundle.getBundle("br.usp.ime.prolog.PrologPluginResources");
		} catch (MissingResourceException x) {
			resourceBundle = null;
		}
	}

	/**
	 * Returns the shared instance.
	 */
	public static PrologPlugin getDefault() {
		return plugin;
	}

	/**
	 * Returns the workspace instance.
	 */
	public static IWorkspace getWorkspace() {
		return ResourcesPlugin.getWorkspace();
	}

	/**
	 * Returns the string from the plugin's resource bundle,
	 * or 'key' if not found.
	 */
	public static String getResourceString(String key) {
		ResourceBundle bundle= PrologPlugin.getDefault().getResourceBundle();
		try {
			return bundle.getString(key);
		} catch (MissingResourceException e) {
			return key;
		}
	}

	/**
	 * Returns the plugin's resource bundle.
	 */
	public ResourceBundle getResourceBundle() {
		return resourceBundle;
	}

	/**
	 *  Sets default values for the preferences fields.
	 */	
	protected void initializeDefaultPreferences(IPreferenceStore store)
	{
	   super.initializeDefaultPreferences(store);

	   // Colors for syntax highlighting.
	   store.setDefault(
		  IPrologPreferenceFields.PREF_COMMENT_COLOR,
		  StringConverter.asString(IPrologColorConstants.COMMENT));
	   store.setDefault(
		  IPrologPreferenceFields.PREF_BUILTIN_COLOR,
		  StringConverter.asString(IPrologColorConstants.BUILTIN));
	   store.setDefault(
		  IPrologPreferenceFields.PREF_DEFAULT_COLOR,
		  StringConverter.asString(IPrologColorConstants.DEFAULT));
	   store.setDefault(
		  IPrologPreferenceFields.PREF_VAR_COLOR,
		  StringConverter.asString(IPrologColorConstants.VAR));
	   store.setDefault(
		  IPrologPreferenceFields.PREF_OPER_COLOR,
		  StringConverter.asString(IPrologColorConstants.OPER));
		store.setDefault(
		   IPrologPreferenceFields.PREF_STRING_COLOR,
		   StringConverter.asString(IPrologColorConstants.STRING));
		   
		// Default path to 'Prolog runtime location'.   			   
		store.setDefault(
			IPrologPreferenceFields.PREF_PROLOG_PATH,
			File.separator + "usr" + File.separator + "bin" + 
			File.separator + "prolog");
	}
	

	/**
	 * Logs the specified status with this plug-in's log.
	 *
	 * @param status status to log.
	 */
	public static void log(final IStatus status) {
		getDefault().getLog().log(status);
	}

	
}
