package br.usp.ime.prolog.launch;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.PlatformObject;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.debug.core.DebugException;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.ILaunch;
import org.eclipse.debug.core.ILaunchConfigurationType;
import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
import org.eclipse.debug.core.ILaunchManager;
import org.eclipse.debug.core.model.IProcess;
import org.eclipse.debug.ui.DebugUITools;

import java.util.ArrayList;

import br.usp.ime.prolog.PrologPlugin;

/** 
 *   Starts the plugins launch configuration.
 *
 */
public class PrologConsole extends PlatformObject {
	/**
	 * Constructor
	 *
	 * @param commands Contains a set of commands to executed by the console.
	 * @throws CoreException If the plugin can't run the launch.
	 * @throws IOException If the supplied file can't be located.
	 */
	public PrologConsole(ArrayList files, String workingDirectory) throws CoreException {
				
		// 'launchConfigurationType'  a launch configuration declarada pelo 
		//  plugin. 
		ILaunchConfigurationType launchConfigurationType =
			DebugPlugin.getDefault().getLaunchManager()
				.getLaunchConfigurationType(IPrologLaunchConstants.PROLOG_LAUNCH_CONFIGURATION_TYPE);
		//candidateConfigsList candidateConfigs = Collections.EMPTY_LIST;
		
		//ILaunchConfiguration[] configs =
		//	DebugPlugin.getDefault().getLaunchManager().getLaunchConfigurations(launchConfigurationType);

		ILaunchConfigurationWorkingCopy launchConfig =
			launchConfigurationType.newInstance(null, DebugPlugin.getDefault()
				.getLaunchManager().generateUniqueLaunchConfigurationNameFrom(
					"Prolog"));

		launchConfig.setAttribute(IPrologLaunchConstants.PROLOG_LAUNCH_FILES_TO_CONSULT, files);
		launchConfig.setAttribute(IPrologLaunchConstants.PROLOG_LAUNCH_WORKING_DIRECTORY, workingDirectory);

		// This msg calls 'PrologLaunchConfigurationDelegate.launch'.
		DebugUITools.launch(launchConfig, ILaunchManager.RUN_MODE);			
	}

	/**
	 * Terminate the running Prolog Console.
	 *
	 */
	public final void terminate() {
		// Kill the Prolog process
		IProcess proc = DebugUITools.getCurrentProcess();
		if (proc != null) {
			ILaunch launch = proc.getLaunch();
			try {
				launch.terminate();
				proc.terminate();
				proc = null;
			} catch (DebugException e) {
				// Log the error to the Eclipse log.
				IStatus status = new Status(IStatus.ERROR,
				   PrologPlugin.getDefault().getDescriptor().getUniqueIdentifier(),0,
				   "Error terminating a running prolog interpreter: " +
				   e.getMessage(), e);
				PrologPlugin.log(status);	   
			}
		}
	}
}