package br.usp.ime.prolog.launch;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
import org.eclipse.debug.ui.AbstractLaunchConfigurationTab;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;

import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IFile;
import org.eclipse.ui.PlatformUI;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IWorkbenchPage; 
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath; 
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;


import br.usp.ime.prolog.PrologPlugin;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.StringTokenizer;

/**
 * @author Juliana
 *
 * Tab-page to the prolog launch configuration.
 *
 */
public class PrologConsoleLaunchTab extends AbstractLaunchConfigurationTab {
	// Project UI widgets.
	private Label fProjLabel;
	private Text fProjText;

	// Files to consult UI widgets.
	private Label fFilesLabel;
	private Text fFilesText;	
	
	private static final String EMPTY_STRING = ""; //$NON-NLS-1$
	

	/* (non-Javadoc)
	 * @see org.eclipse.debug.ui.ILaunchConfigurationTab#createControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createControl(Composite parent) {		
		Font font = parent.getFont();
		
		Composite comp = new Composite(parent, SWT.NONE);
		setControl(comp);		
		GridLayout topLayout = new GridLayout();
		comp.setLayout(topLayout);		
		GridData gd;
		
		createVerticalSpacer(comp);
		
		Composite projComp = new Composite(comp, SWT.NONE);
		GridLayout projLayout = new GridLayout();
		projLayout.numColumns = 3;
		projLayout.marginHeight = 0;
		projLayout.marginWidth = 0;
		projComp.setLayout(projLayout);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		projComp.setLayoutData(gd);
		projComp.setFont(font);
		
		fProjLabel = new Label(projComp, SWT.NONE);
		//fProjLabel.setText(LauncherMessages.getString("appletlauncher.maintab.projectlabel.name")); //$NON-NLS-1$
		fProjLabel.setText("Project: ");
		gd = new GridData();
		gd.horizontalSpan = 3;
		fProjLabel.setLayoutData(gd);
		fProjLabel.setFont(font);
		
		fProjText = new Text(projComp, SWT.SINGLE | SWT.BORDER);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		fProjText.setLayoutData(gd);
		fProjText.setFont(font);
		this.fProjText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent evt) {
				updateLaunchConfigurationDialog();
			}
		});
		
				
		Label spacer = createVerticalSpacer(projComp);
		gd = new GridData();
		gd.horizontalSpan = 3;
		spacer.setLayoutData(gd);
		
		fFilesLabel = new Label(projComp, SWT.NONE);
		//fMainLabel.setText(LauncherMessages.getString("appletlauncher.maintab.mainclasslabel.name")); //$NON-NLS-1$
		fFilesLabel.setText("Files to consult (in single quotes, comma delimited):");
		gd = new GridData();
		gd.horizontalSpan = 3;
		fFilesLabel.setLayoutData(gd);
		fFilesLabel.setFont(font);

		fFilesText = new Text(projComp, SWT.SINGLE | SWT.BORDER);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		fFilesText.setLayoutData(gd);
		fFilesText.setFont(font);
		this.fFilesText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent evt) {
				updateLaunchConfigurationDialog();
			}
		});
		
	}

	/**
	 * Creates some empty space. 
	 */
	private Label createVerticalSpacer(Composite comp) {
		return new Label(comp, SWT.NONE);
	}


	/* (non-Javadoc)
	 * @see org.eclipse.debug.ui.ILaunchConfigurationTab#setDefaults(org.eclipse.debug.core.ILaunchConfigurationWorkingCopy)
	 */
	public void setDefaults(ILaunchConfigurationWorkingCopy config) {	
		IProject project = getSelectedProject(config);	
		initializeProject(project, config);
		initializeFiles(project, config);		
	}

	/**
	 * Sets the 'project's name and its location as 'config's attributes.
	 *  
	 * @param project
	 * @param config
	 */
	private void initializeProject(IProject project, ILaunchConfigurationWorkingCopy config) {
		String projectName = EMPTY_STRING;		
		if (project != null) {			
			projectName = project.getName();
			setWorkingDirectory(project, config);
		}
		config.setAttribute(IPrologLaunchConstants.PROLOG_LAUNCH_PROJECT, projectName);												
	}

	/**
	 * Creates a string with all the names of prolog files in 'project' and
	 * sets it as a 'config's attribute.
	 * 
	 * @param project
	 * @param config
	 */
	private void initializeFiles(IProject project, ILaunchConfigurationWorkingCopy config) {
		String fieldFiles = EMPTY_STRING;
		ArrayList files = new ArrayList();
		if (project != null) {
			try {
				IResource[] resources = project.members();
				for (int i=0; i<resources.length; i++) {
					IResource r = resources[i];
					if (r instanceof IFile) {
						IFile f = (IFile) r;
						for (int j=0; j<PrologPlugin.PROLOG_EXTENSIONS.length; j++) {
							if (f.getFileExtension().equals(PrologPlugin.PROLOG_EXTENSIONS[j])) {
								files.add(f);
								break;
							}
						}
					}					
				}	
				fieldFiles = buildStringFiles(files);
			}
			catch (CoreException ce) {
				// Log the error to the Eclipse log.
				IStatus status = new Status(IStatus.ERROR,
				   PrologPlugin.getDefault().getDescriptor().getUniqueIdentifier(),0,
				   "Error in PrologConsoleLaunchTab.initializeFiles: " +
				   ce.getMessage(), ce);
				PrologPlugin.log(status);	   
			}						
		}
		config.setAttribute(IPrologLaunchConstants.PROLOG_LAUNCH_STRING_FILES, fieldFiles);
	}

	/**
	 * Builds a comma delimited String with de filenames in 'files'.
	 * 
	 * @param files
	 */
	private String buildStringFiles(ArrayList files) {		
		String field = EMPTY_STRING;		
		Iterator it = files.iterator();
		while (it.hasNext()) {
			Object obj = it.next();
			if (obj instanceof IFile) {
				IFile file = (IFile) obj;
				field += "'" + file.getName() + "'";
				if (it.hasNext()) {
					field += ", ";
				}
			}
		}	
		return field;
	}

	/** 
	 * Gets the current project selected in the workbench.
	 * 
	 * @param config
	 * @return the current project selected in the workbench.
	 */
	private IProject getSelectedProject(ILaunchConfigurationWorkingCopy config) {
		IWorkbenchWindow window = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
		if (window != null) {
			IWorkbenchPage page = window.getActivePage();
			if (page != null) {
				ISelection selection = page.getSelection();
				if (selection instanceof IStructuredSelection) {
					IStructuredSelection ss = (IStructuredSelection)selection;
					if (!ss.isEmpty()) {
						Object obj = ss.getFirstElement();
						if (obj instanceof IResource) {
							IResource i = (IResource) obj;
							IProject pro = i.getProject();
							return pro;														
						}
					}
				}
				// If the editor has the focus...
				IEditorPart part = page.getActiveEditor();
				if (part != null) {
					IEditorInput input = part.getEditorInput();
					IFile file = (IFile) input.getAdapter(IFile.class);
					return file.getProject();
				}				
			}
		}
		return null;							
	}

	/* (non-Javadoc)
	 * @see org.eclipse.debug.ui.ILaunchConfigurationTab#initializeFrom(org.eclipse.debug.core.ILaunchConfiguration)
	 */
	public void initializeFrom(ILaunchConfiguration config) {
		updateProjectFromConfig(config);
		updateFilesFromConfig(config);
	}

	/**
	 * Updates the project name field, acording to a launch
	 * configuration 'config' attribute.
	 * 
	 * @param config
	 */
	private void updateProjectFromConfig(ILaunchConfiguration config) {		
		try {
			String projectName = config.getAttribute(IPrologLaunchConstants.PROLOG_LAUNCH_PROJECT, EMPTY_STRING);
			fProjText.setText(projectName);	
		} catch (CoreException ce) {
			// Log the error to the Eclipse log.
			IStatus status = new Status(IStatus.ERROR,
			   PrologPlugin.getDefault().getDescriptor().getUniqueIdentifier(),0,
			   "Error in PrologConsoleLaunchTab.updateProjectFromConfig: " +
			   ce.getMessage(), ce);
			PrologPlugin.log(status);	   
		}
	}	


	/**
	 * Updates the 'Files to consult' field, according to a launch
	 * configuration 'config' attribute. 
	 * 
	 * @param config
	 */
	private void updateFilesFromConfig(ILaunchConfiguration config) {
		try {
			String files = config.getAttribute(IPrologLaunchConstants.PROLOG_LAUNCH_STRING_FILES, EMPTY_STRING);
			fFilesText.setText(files);
		} catch (CoreException ce) {
			// Log the error to the Eclipse log.
			IStatus status = new Status(IStatus.ERROR,
			   PrologPlugin.getDefault().getDescriptor().getUniqueIdentifier(),0,
			   "Error in PrologConsoleLaunchTab.updateFilesFromConfig: " +
			   ce.getMessage(), ce);
			PrologPlugin.log(status);	   
		}
	}	


	/* (non-Javadoc)
	 * @see org.eclipse.debug.ui.ILaunchConfigurationTab#performApply(org.eclipse.debug.core.ILaunchConfigurationWorkingCopy)
	 */
	public void performApply(ILaunchConfigurationWorkingCopy config) {
		config.setAttribute(IPrologLaunchConstants.PROLOG_LAUNCH_PROJECT, fProjText.getText());
		config.setAttribute(IPrologLaunchConstants.PROLOG_LAUNCH_STRING_FILES, fFilesText.getText());
		
		// TODO: Verificar se projeto e arquivos contidos nos campos da tab
		//       page ainda so vlidos.
		
		setRunAttributes(config);			
						
	}

	/**
	 * Sets the launch configuration <code>config</code> attributes used to run
	 * the prolog console, according to other attributes
	 * already defined.  
	 * 
	 * @param config
	 */
	private void setRunAttributes(ILaunchConfigurationWorkingCopy config) {
		try {
			String projectName = config.getAttribute(IPrologLaunchConstants.PROLOG_LAUNCH_PROJECT, EMPTY_STRING);
		
			if (projectName != EMPTY_STRING) {
				IProjectDescription p = ResourcesPlugin.getWorkspace().newProjectDescription(projectName);
				IPath projectPath = p.getLocation();
				setFilesToConsult(config);
			}
			
		}
		catch (CoreException ce) {
			// Log the error to the Eclipse log.
			IStatus status = new Status(IStatus.ERROR,
			   PrologPlugin.getDefault().getDescriptor().getUniqueIdentifier(),0,
			   "Error in PrologConsoleLaunchTab.setRunAttributes: " +
			   ce.getMessage(), ce);
			PrologPlugin.log(status);	 
		}  		
	}

	/**
	 * Sets the 'IPrologLaunchConstants.PROLOG_LAUNCH_FILES_TO_CONSULT' <code>config</code>'s 
	 * attribute, used to consult prolog files into prolog console.
	 * This attribute is defined according to the 'Files to consult' field, whose
	 * contents are stored in the 'IPrologLaunchConstants.PROLOG_LAUNCH_STRING_FILES'
	 * 'config's attribute.
	 * 
	 * @param config
	 */
	private void setFilesToConsult(ILaunchConfigurationWorkingCopy config) {
		try {
			ArrayList files = new ArrayList();
			String prologFiles = config.getAttribute(IPrologLaunchConstants.PROLOG_LAUNCH_STRING_FILES, EMPTY_STRING);
			prologFiles = prologFiles.trim();
			
			StringTokenizer filesTokenizer = new StringTokenizer(prologFiles,",");
		
			while (filesTokenizer.hasMoreTokens()) {
				String prologFile = filesTokenizer.nextToken().trim();
				if (prologFile.startsWith("'") && prologFile.endsWith("'") && prologFile.length() > 1) {								
					prologFile = prologFile.substring(1,prologFile.length()-1);
				}			
				files.add(prologFile);						
			}	
			config.setAttribute(IPrologLaunchConstants.PROLOG_LAUNCH_FILES_TO_CONSULT, files);											
		}
		catch (CoreException ce) {
			// Log the error to the Eclipse log.
			IStatus status = new Status(IStatus.ERROR,
			   PrologPlugin.getDefault().getDescriptor().getUniqueIdentifier(),0,
			   "Error in PrologConsoleLaunchTab.setFilesToConsult: " +
			   ce.getMessage(), ce);
			PrologPlugin.log(status);	   
		}
	}


	/**
	 * Set the 'IPrologLaunchConstants.PROLOG_LAUNCH_WORKING_DIRECTORY'
	 * 'config's attribute as the 'project' location.
	 * 
	 * @param project
	 * @param config
	 */
	private void setWorkingDirectory(IProject project, ILaunchConfigurationWorkingCopy config) {
		String workingDirectory = project.getLocation().toString();
		if (workingDirectory != "" && workingDirectory != null) {
			workingDirectory += IPath.SEPARATOR;
		}
		config.setAttribute(IPrologLaunchConstants.PROLOG_LAUNCH_WORKING_DIRECTORY,workingDirectory);
	}


	/* (non-Javadoc)
	 * @see org.eclipse.debug.ui.ILaunchConfigurationTab#getName()
	 */
	public String getName() {		
		return "Main";
	}

}
