package br.usp.ime.prolog.launch;

import java.text.MessageFormat;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;

import org.eclipse.debug.core.ILaunch;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.model.IProcess;
import org.eclipse.debug.core.model.ILaunchConfigurationDelegate;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status; 
 
import br.usp.ime.prolog.PrologPlugin;
import br.usp.ime.prolog.preferences.IPrologPreferenceFields;

import java.io.File;
import java.io.IOException;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;

/**
 *  Launchs a system processes (the Prolog interpreter).
 *
 * @see ILaunch
 * @see ILaunchManager
 */
public class PrologLaunchConfigurationDelegate implements ILaunchConfigurationDelegate {
	private ILaunchConfiguration fElement;
	private ILaunch fLaunch;
	private String fMode;
	private String fPrologRuntime;
	private String fPrologCommand;
	private Process fInferiorProcess;
	private IProcess fEclipseProcHandle;
	private IProgressMonitor fMonitor;

	/**
	 * Constructor
	 *
	 */
	public PrologLaunchConfigurationDelegate() {
		// Get the prolog runtime path from the preference page.
		fPrologRuntime = PrologPlugin.getDefault().getPreferenceStore()
								.getString(IPrologPreferenceFields.PREF_PROLOG_PATH);
								
		fPrologCommand = "";
	}

	/**
	 *   This method actually launches the Prolog Console.
	 *
	 * @param configuration the launch configuration.
	 * @param mode The launch mode.
	 * @param launch The launch event.
	 * @param monitor The progress monitor.
	 * @throws CoreException if the launch fails.
	 */
	public final void launch(
	    final ILaunchConfiguration configuration,
		final String mode,
		final ILaunch launch,
		final IProgressMonitor monitor)
		throws CoreException {

		try {

			fPrologRuntime =
				PrologPlugin.getDefault().getPreferenceStore().getString(
					IPrologPreferenceFields.PREF_PROLOG_PATH);

			//File prologFile = new File(fPrologRuntime);
			//if (!prologFile.isFile()) {
				// Notify the user.			
			//}

			if (monitor == null) {
				fMonitor = new NullProgressMonitor();
			} 
			else {
				fMonitor = monitor;
			}

			fElement = configuration;
			fMode = mode;
			fLaunch = launch;

			monitor.beginTask(MessageFormat.format("Launching {0}...", new String[] {configuration.getName()}), 3); //$NON-NLS-1$
			if (monitor.isCanceled()) {
				// Was the process canceled?
				return;
			}

			//IDebugEventFilter filter = new PrologInferiorEventFilter(launch);
			//DebugPlugin.getDefault().addDebugEventFilter(filter);

			// Recuperar a lista de comandos, definida como atributo da
			// LaunchConfiguration.
			List files =
				configuration.getAttribute(
					IPrologLaunchConstants.PROLOG_LAUNCH_FILES_TO_CONSULT,
					new ArrayList());
			
			// Recuperar o diretrio de trabalho, tambm definido como atributo
			// da LaunchConfiguration.
			String workingDirectory = 
				configuration.getAttribute(
					IPrologLaunchConstants.PROLOG_LAUNCH_WORKING_DIRECTORY,
					"");		
			File fworkingDirectory = null;
			
			if (workingDirectory != "" && workingDirectory != null) {
				fworkingDirectory = new File(workingDirectory);
			}
					
			// Executar o console prolog.
			fInferiorProcess =
				DebugPlugin.exec(new String[] { fPrologRuntime }, fworkingDirectory);

			// Ligar o processo ao console de debug (?)
			fEclipseProcHandle =  DebugPlugin.newProcess(launch,
			    fInferiorProcess,"Prolog Session");

			// "Passar" os comandos para o processo.
			writeCommands(fEclipseProcHandle, files, workingDirectory);

			monitor.worked(1);
		}
		catch (CoreException e) {
			// Log the error to the Eclipse log.
			IStatus status = new Status(IStatus.ERROR,PrologPlugin.getDefault().getDescriptor().getUniqueIdentifier(),0,
				"Problems executing prolog interpreter.\n " +
				"Prolog interpreter path may be incorrect! " + 
				"Check the 'Prolog Preferences'.", e);
			throw new CoreException(status);				
		}
	}

	
	/**
	 * Writes the consult commands to the prolog interpreter process.
	 * 
	 * @param feclipseProcHandle the prolog interpreter process.
	 * @param files the files to be consulted.
	 * @param fDirectory the working directory.
	 */
	private void writeCommands(IProcess feclipseProcHandle, List files, String fDirectory) {		
		Iterator it = files.iterator();
		while (it.hasNext()) {
			String file = (String) it.next();
			if (fDirectory != "" && fDirectory != null) {
				file = fDirectory + file;
			}
			String command = "consult('" + file + "').\n";
			try {
				fEclipseProcHandle.getStreamsProxy().write(command);
			}
			catch (IOException ioe) {
				// Log the error to the Eclipse log.
				IStatus status = new Status(IStatus.ERROR,
				   PrologPlugin.getDefault().getDescriptor().getUniqueIdentifier(),0,
				   "Problems sending 'consult' command to the prolog interpreter process.", ioe);
				PrologPlugin.log(status);	   
			}				
		}		
	}
}

